@@ -1,5 +1,25 @@
 <?php
 
+/**
+ * SSH/FTP Log Monitor Manager
+ * 
+ * Monitors SSH and FTP log files for failed login attempts and automatically
+ * blocks IPs that exceed configured thresholds using service-level or 
+ * iptables-based blocking methods.
+ * 
+ * Features:
+ * - Real-time log parsing with position tracking
+ * - High-performance in-memory caching with SQLite persistence  
+ * - Service-level blocking via SSH/FTP configuration includes
+ * - Configurable thresholds and block durations per service
+ * - Exemption list integration
+ * - Multiple blocking methods (service-level vs iptables)
+ * 
+ * @version 2025.08.73
+ * @requires PHP 8.2+, SQLite3, systemctl access
+ * @author WHP Development Team
+ * @package WHP\Security
+ */
 class log_monitor_manager {
     
     private $db_path = '/docker/whp/sql/security.db';
@@ -9,6 +29,16 @@ class log_monitor_manager {
     private $attempts_cache = array(); // In-memory cache for performance
     private $last_cache_save = 0;
     
+    /**
+     * Constructor - Initialize database and configuration
+     * 
+     * Sets up SQLite database connection, creates required tables if they don't exist,
+     * loads default configuration values, and initializes in-memory cache for
+     * high-performance log monitoring operations.
+     * 
+     * @throws Exception If database connection or table creation fails
+     * @since 2025.08.73
+     */
     public function __construct() {
         $this->init_database();
         $this->load_config();
@@ -98,8 +128,8 @@ class log_monitor_manager {
             'ssh_block_duration' => ['3600', 'SSH IP block duration in seconds (1 hour)'],
             'ftp_block_duration' => ['3600', 'FTP IP block duration in seconds (1 hour)'],
             'log_check_interval' => ['300', 'How often to check logs in seconds (5 minutes)'],
-            'ssh_log_file' => ['/var/log/auth.log', 'SSH/Auth log file path'],
-            'ftp_log_file' => ['/var/log/proftpd/proftpd.log', 'FTP log file path'],
+            'ssh_log_file' => ['/var/log/secure', 'SSH/Auth log file path'],
+            'ftp_log_file' => ['/var/log/proftpd/auth.log', 'FTP log file path'],
             'blocking_method' => ['service', 'IP blocking method (service=service-level, iptables=firewall-level)'],
             'docker_network_preserve' => ['1', 'Preserve Docker network access when blocking IPs']
         ];
@@ -120,6 +150,21 @@ class log_monitor_manager {
         }
     }
     
+    /**
+     * Retrieves a configuration value with optional default
+     * 
+     * Looks up configuration value from in-memory cache first, then database
+     * if not cached. Returns the default value if the key is not found.
+     * 
+     * @param string $key Configuration key name
+     * @param mixed $default Default value to return if key not found
+     * @return string|mixed Configuration value or default value
+     * @since 2025.08.73
+     * 
+     * @example
+     * $max_attempts = $monitor->get_config('max_ssh_attempts', '5');
+     * $log_file = $monitor->get_config('ssh_log_file', '/var/log/auth.log');
+     */
     public function get_config($key, $default = null) {
         return isset($this->config[$key]) ? $this->config[$key] : $default;
     }
@@ -154,6 +199,23 @@ class log_monitor_manager {
         return false;
     }
     
+    /**
+     * Sets a configuration value with validation and persistence
+     * 
+     * Validates the configuration key against allowed keys, performs value-specific
+     * validation, and stores the setting in the database. All configuration changes
+     * are logged for audit purposes.
+     * 
+     * @param string $key Configuration key name (must be in allowed keys list)
+     * @param string $value Configuration value (validated based on key type)
+     * @return bool True on success, false on validation failure or database error
+     * @throws InvalidArgumentException If key is not allowed or value is invalid
+     * @since 2025.08.73
+     * 
+     * @example
+     * $monitor->set_config('max_ssh_attempts', '10');
+     * $monitor->set_config('blocking_method', 'service');
+     */
     public function set_config($key, $value) {
         // Validate log file paths before saving
         if (in_array($key, ['ssh_log_file', 'ftp_log_file'])) {
@@ -197,6 +259,22 @@ class log_monitor_manager {
         return $config;
     }
     
+    /**
+     * Main log monitoring entry point
+     * 
+     * Scans configured log files for new failed login attempts and applies blocking rules
+     * based on configured thresholds. Processes both SSH and FTP logs incrementally,
+     * tracking file positions to avoid reprocessing entries.
+     * 
+     * @return array Statistics about processed attempts and blocks applied
+     * @throws Exception If database operations fail or log files cannot be accessed
+     * @since 2025.08.73
+     * 
+     * @example
+     * $monitor = new log_monitor_manager();
+     * $stats = $monitor->check_logs();
+     * echo "Processed {$stats['ssh_attempts']} SSH attempts";
+     */
     public function check_logs() {
         if ($this->get_config('ssh_monitoring_enabled') == '1') {
             $this->check_ssh_logs();
@@ -211,7 +289,7 @@ class log_monitor_manager {
     }
     
     private function check_ssh_logs() {
-        $log_file = $this->get_config('ssh_log_file', '/var/log/auth.log');
+        $log_file = $this->get_config('ssh_log_file', '/var/log/secure');
         
         // Validate log file path to prevent path traversal
         if (!$this->is_safe_log_path($log_file)) {
@@ -233,7 +311,7 @@ class log_monitor_manager {
     }
     
     private function check_ftp_logs() {
-        $log_file = $this->get_config('ftp_log_file', '/var/log/proftpd/proftpd.log');
+        $log_file = $this->get_config('ftp_log_file', '/var/log/proftpd/auth.log');
         
         // Validate log file path to prevent path traversal
         if (!$this->is_safe_log_path($log_file)) {
@@ -264,11 +342,48 @@ class log_monitor_manager {
             $current_inode = fileinode($log_file);
             // If inode changed, file was rotated, start from beginning
             if ($current_inode != $row['last_inode']) {
-                return 0;
+                return $this->get_safe_starting_position($log_file);
             }
             return $row['last_position'];
         }
         
+        // No position stored - for large files, start near the end
+        return $this->get_safe_starting_position($log_file);
+    }
+    
+    /**
+     * Get a safe starting position for large log files
+     * 
+     * For files larger than 1MB, starts from near the end (last 100KB)
+     * to avoid memory exhaustion when processing very large log files.
+     * 
+     * @param string $log_file Path to the log file
+     * @return int Starting position in bytes
+     * @since 2025.08.73
+     */
+    private function get_safe_starting_position($log_file) {
+        $file_size = filesize($log_file);
+        
+        // For files larger than 1MB, start from last 100KB
+        if ($file_size > 1048576) { // 1MB = 1048576 bytes
+            // Start 100KB from the end (or from beginning if file is smaller)
+            $safe_position = max(0, $file_size - 102400); // 100KB = 102400 bytes
+            
+            // Try to find a line break near our position to avoid partial lines
+            $file = fopen($log_file, 'r');
+            if ($file) {
+                fseek($file, $safe_position);
+                // Read and discard partial line
+                fgets($file);
+                $aligned_position = ftell($file);
+                fclose($file);
+                return $aligned_position;
+            }
+            
+            return $safe_position;
+        }
+        
+        // For smaller files, start from the beginning
         return 0;
     }
     
@@ -292,8 +407,13 @@ class log_monitor_manager {
         
         fseek($file, $start_position);
         
-        while (($line = fgets($file)) !== false) {
+        // Limit to 10000 lines per run to prevent memory issues
+        $max_lines = 10000;
+        $line_count = 0;
+        
+        while (($line = fgets($file)) !== false && $line_count < $max_lines) {
             $lines[] = trim($line);
+            $line_count++;
         }
         
         $new_position = ftell($file);
@@ -301,6 +421,11 @@ class log_monitor_manager {
         
         $this->update_log_position($log_file, $new_position);
         
+        // Log a warning if we hit the limit
+        if ($line_count >= $max_lines) {
+            error_log("WHP Log Monitor: Processed maximum of $max_lines lines from $log_file. Will continue in next run.");
+        }
+        
         return $lines;
     }
     
@@ -310,22 +435,34 @@ class log_monitor_manager {
             return;
         }
         
-        // SSH failed login patterns
+        // SSH failed login patterns for /var/log/secure format
         $patterns = [
-            // SSH password authentication failed
-            '/Failed password for (?:invalid user )?(\S+) from (\d+\.\d+\.\d+\.\d+)/',
-            // SSH connection closed
+            // SSH password authentication failed (with or without "invalid user")
+            '/Failed password for (?:invalid user )?(\S+) from (\d+\.\d+\.\d+\.\d+) port \d+ ssh2/',
+            // Invalid user attempt
+            '/Invalid user (\S+) from (\d+\.\d+\.\d+\.\d+) port \d+/',
+            // PAM authentication failure
+            '/pam_unix\(sshd:auth\): authentication failure;.*rhost=(\d+\.\d+\.\d+\.\d+)(?:\s+user=(\S+))?/',
+            // Connection closed (potential brute force)
             '/Connection closed by (\d+\.\d+\.\d+\.\d+) port \d+/',
-            // Invalid user
-            '/Invalid user (\S+) from (\d+\.\d+\.\d+\.\d+)/',
-            // Authentication failure
-            '/authentication failure.*rhost=(\d+\.\d+\.\d+\.\d+).*user=(\S+)/',
         ];
         
         foreach ($patterns as $pattern) {
             if (preg_match($pattern, $line, $matches)) {
-                $username = isset($matches[2]) ? $matches[2] : (isset($matches[1]) && !filter_var($matches[1], FILTER_VALIDATE_IP) ? $matches[1] : 'unknown');
-                $ip = isset($matches[2]) && filter_var($matches[2], FILTER_VALIDATE_IP) ? $matches[2] : $matches[1];
+                // Determine IP and username based on pattern matched
+                if (strpos($line, 'pam_unix') !== false) {
+                    // PAM pattern: IP is in $matches[1], username in $matches[2] (if present)
+                    $ip = $matches[1];
+                    $username = isset($matches[2]) && !empty($matches[2]) ? $matches[2] : 'unknown';
+                } elseif (strpos($line, 'Connection closed') !== false) {
+                    // Connection closed: IP is in $matches[1], no username
+                    $ip = $matches[1];
+                    $username = 'unknown';
+                } else {
+                    // Failed password or Invalid user: username in $matches[1], IP in $matches[2]
+                    $username = $matches[1];
+                    $ip = $matches[2];
+                }
                 
                 // Validate extracted data
                 if (filter_var($ip, FILTER_VALIDATE_IP)) {
@@ -348,30 +485,24 @@ class log_monitor_manager {
             return;
         }
         
-        // FTP failed login patterns
-        $patterns = [
-            // ProFTPD authentication failed
-            '/USER (\S+).*\[(\d+\.\d+\.\d+\.\d+)\].*FAIL/',
-            '/authentication failed.*user (\S+).*rhost \[(\d+\.\d+\.\d+\.\d+)\]/',
-            '/Login incorrect.*user (\S+).*\[(\d+\.\d+\.\d+\.\d+)\]/',
-        ];
+        // ProFTPD auth.log format for failed logins
+        // Format: hostname IP_ADDRESS [username] [timestamp] "PASS (hidden)" 530 -
+        // Status code 530 means authentication failed
         
-        foreach ($patterns as $pattern) {
-            if (preg_match($pattern, $line, $matches)) {
-                $username = $matches[1];
-                $ip = $matches[2];
-                
-                // Validate extracted data
-                if (filter_var($ip, FILTER_VALIDATE_IP)) {
-                    // Sanitize username (allow alphanumeric, underscore, dash, max 32 chars)
-                    $username = substr(preg_replace('/[^a-zA-Z0-9_-]/', '', $username), 0, 32);
-                    if (empty($username)) {
-                        $username = 'unknown';
-                    }
-                    
-                    $this->record_service_failed_attempt('ftp', $ip, $username, $line);
+        // Match failed password attempts (status code 530)
+        if (preg_match('/^(\S+)\s+(\d+\.\d+\.\d+\.\d+)\s+(?:(\S+)\s+)?.*"PASS \(hidden\)" 530/', $line, $matches)) {
+            $ip = $matches[2];
+            $username = isset($matches[3]) && $matches[3] !== '-' && $matches[3] !== 'UNKNOWN' ? $matches[3] : 'unknown';
+            
+            // Validate extracted data
+            if (filter_var($ip, FILTER_VALIDATE_IP)) {
+                // Sanitize username (allow alphanumeric, underscore, dash, max 32 chars)
+                $username = substr(preg_replace('/[^a-zA-Z0-9_-]/', '', $username), 0, 32);
+                if (empty($username)) {
+                    $username = 'unknown';
                 }
-                break;
+                
+                $this->record_service_failed_attempt('ftp', $ip, $username, $line);
             }
         }
     }
@@ -476,6 +607,25 @@ class log_monitor_manager {
         }
     }
     
+    /**
+     * Blocks an IP address for a specific service
+     * 
+     * Applies blocking using the configured method (service-level or iptables).
+     * Checks exemption list before blocking and validates all inputs. Updates
+     * service configuration files and reloads services gracefully.
+     * 
+     * @param string $service Service name ('ssh' or 'ftp')
+     * @param string $ip IP address to block (IPv4 format)
+     * @param string $reason Human-readable reason for blocking
+     * @param int $failed_attempts Number of failed attempts that triggered block
+     * @return bool True on success, false on failure or if IP is exempted
+     * @throws InvalidArgumentException If service or IP format is invalid
+     * @since 2025.08.73
+     * 
+     * @example
+     * $result = $monitor->block_service_ip('ssh', '203.0.113.45', 'Too many failed attempts', 15);
+     * if ($result) echo "IP blocked successfully";
+     */
     public function block_service_ip($service, $ip, $reason = 'Manual block', $failed_attempts = 0) {
         // Check if IP is whitelisted
         require_once('/docker/whp/web/libs/security_manager.php');
@@ -512,6 +662,23 @@ class log_monitor_manager {
         return true;
     }
     
+    /**
+     * Removes an IP address from service-specific block list
+     * 
+     * Removes blocking rules using the same method that was used to apply them.
+     * Updates service configuration files and reloads services gracefully.
+     * Also clears failed attempt records for the IP.
+     * 
+     * @param string $service Service name ('ssh' or 'ftp')
+     * @param string $ip IP address to unblock (IPv4 format)
+     * @return bool True on success, false on failure
+     * @throws InvalidArgumentException If service or IP format is invalid
+     * @since 2025.08.73
+     * 
+     * @example
+     * $result = $monitor->unblock_service_ip('ssh', '203.0.113.45');
+     * if ($result) echo "IP unblocked successfully";
+     */
     public function unblock_service_ip($service, $ip) {
         // Check what type of blocking was used
         $stmt = $this->db->prepare('SELECT firewall_rule_added FROM service_blocked_ips WHERE service_type = ? AND ip_address = ?');
